@@ -1,5 +1,4 @@
 from .state import State
 from .keyquery import KeyQuery
 from .attributename import AttributeName
-from .constants import ErrorMessages
-from .methods import Methods
+from .methods import Methods, ErrorMessages
